/*********************************************************\
*  AVR MegaSquirt - 2003                                  *
*  (C) 2003 [Michael Kristensen, mik@caffrey.dk]          *
***********************************************************
*  -- History --                                          *
*    release 0.1 - 06-03-2003                             *
*            http://caffrey.dk/megasquirt                 *
*    initial Motorola version by Bowling & Grippo (2002)  *
*            http://www.bgsoflex.com/megasquirt.html      *
\*********************************************************/

#ifndef COMM_H
#define COMM_H

#include <inttypes.h>
#include <avr/io.h>
// see TABLES_TO_DUMP1 before changing this
#define CMD_QUEUE_SIZE 9

enum MegatunePageIdx
{
    MT_PAGE_IDX_CONFIG = 0,     // config
    MT_PAGE_IDX_HWMAP,          // hw map
    MT_PAGE_IDX_BOOST,          // boost structure
    MT_PAGE_IDX_CONFIGEXT,      // config_ext
    MT_PAGE_IDX_TEMPERATURE,    // ve.mat_enrich table
    MT_PAGE_IDX_A340E,          // at structure
    MT_PAGE_IDX_LAUNCHRPM,	// launchrpm table

/* following tables all have 2 axis in them (usually RPM and KPA).
* Table size must be specified (8x8, not 8x8+8+8),
* get_bins_pointer() decides which pages share bins => ranges are sent automagically.
* Could be changed, so only LAMBDAVIEW is included and everything after ...IGNADV
*/
    PAGE_IDX_FIRST_RPMKPA,
    // labda size
    MT_PAGE_IDX_LAMBDA = PAGE_IDX_FIRST_RPMKPA, // labmda (lambda_size default 8x8)
    // LAMBDAVIEW is same data as LAMBDA, but 1/x style for retarded tuningprograms
    MT_PAGE_IDX_LAMBDAVIEW,         // labmda (lambda_size default 8x8)
    // ve size
    MT_PAGE_IDX_VE,             	// ve (ve_size default 16x14 in my_make)
    MT_PAGE_IDX_CHARGE,
    MT_PAGE_IDX_MAPCORR,            // map_corr (map_corr_size default 8x8)
    MT_PAGE_IDX_MATRET,

// following tables all share rpmkpa with ign_adv table:
// ign size
    MT_PAGE_IDX_IGNADV,             // ign_adv (adv_size default 12x12)
    MT_PAGE_IDX_BOOST_REF,          // boost ref (adv_size default 12x12)
    MT_PAGE_IDX_1JZVVTI,            // vvti_adv (adv_size default 12x12)
    MT_PAGE_IDX_KNOCK,              // knock ref (adv_size default 12x12)

// please UNDERSTAND page_has_rpmkpa before choosing index for a new table (likely must be placed before VE)
// UPDATE this when adding new table:
    MT_PAGE_IDX_LAST = MT_PAGE_IDX_KNOCK
};

#define alternative_rtdata()	(config.config12&_BV(ALTERNATIVE_RT_DATA))

#define MT_PAGE_IDX_DUMMY (0xFF)
#define MT_PAGE_SIZE_DUMMY (0)

// only pages that have idx greater or equal MT_PAGE_IDX_VE clone the rpm and kpa axis of the VE table
#define page_has_rpmkpa(pageidx) (pageidx >= PAGE_IDX_FIRST_RPMKPA)

#define MT_CODEVER_LEN (26)

struct privch_t {
	uint8_t a;
	uint8_t b;
};

struct bins_pointer_t {
	uint8_t *p_kpa_range, *p_rpm_range;
	uint8_t kpa_size, rpm_size;
};

struct cmd_queue_t {
  uint8_t (*func)(uint8_t i);	// null is special, means send bytes from (SRAM) private[]
  uint8_t len;    // number of bytes to send
  uint8_t count;  // number of bytes already sent
  union {
    uint8_t * private;
    uint8_t (*privatefunc)(uint8_t i);
  };

  union {
  	struct privch_t privch;
   	// direct access of privw (eg. using privch) is endian-dependent!!!
   	// Do NOT access privw using privch.
   	// Only use privch in case when not accessed as privw
   	// http://www.avrfreaks.net/index.php?module=FreaksFiles&func=viewFile&id=271&showinfo=1
   	// use LO() and HI() macros
	uint16_t privw;
  };
	uint8_t seq;   // sequence number used in sipr
};

#define LO(x)	(x & 0xFF)
#define HI(x)	((x >> 8) & 0xFF)

#define MT_LAMBDA_OFFSET 73
#define LAMBDA_INT_OFFSET 200


// We could enumerate instead, but ansi-C promotes enumerations to
// integers (16 bit), no need for that!
#define CMD    0
#define OFFSET 1
#define VALUE  2
extern uint8_t comm_state;

extern uint8_t cmd_queue_head, cmd_queue_tail;
extern struct cmd_queue_t cmd_queue[CMD_QUEUE_SIZE];

#define comm_buffer_full() (cmd_queue_head == ((cmd_queue_tail + 1) % CMD_QUEUE_SIZE))

#if ((defined WBO2) || (defined KNOCK_8101))
#define comm_TX_plain_SIZE (64)
extern uint8_t comm_TX_plainbytes[comm_TX_plain_SIZE];
#endif

extern uint8_t comm1_relay_allowed;

void initUART(void);
void initUART1(void);

void comm(uint8_t data);

// keep MT_RTVAR_COUNT a multiples of 4
// (optional filler at the end) so megatune compatibility
// is not broken so often
#define MT_RTVAR_COUNT (56)
#define MT_RTVAR_COUNTMORE (64)
uint8_t MTsendRTvar(uint8_t i);
uint8_t MTtestComm(uint8_t i);
uint8_t MTcodeVer(uint8_t i);

void change_page(uint8_t page);
void get_bins_pointer(uint8_t page, struct bins_pointer_t *bins_pointer);
uint8_t send_page(uint8_t i);
void store_page(uint8_t addr, uint8_t value);
uint8_t sendConfigVar(uint8_t i);
uint8_t allSensorRaw(uint8_t i);
uint8_t ConfigDump(uint8_t cnt);
uint8_t TableDump(uint8_t cnt);
uint8_t pushfunc(uint8_t (*f)(uint8_t i), uint8_t len);
void pushfuncpar2(uint8_t (*f)(uint8_t i), uint8_t len, void * par, uint8_t p2);
void pushfuncpar(uint8_t (*f)(uint8_t i), uint8_t len, void *);
void pushfuncTableDump(uint8_t *p, uint8_t entries);
uint8_t pushfuncsipr(uint8_t (*f)(uint8_t i), uint8_t len, uint8_t seq);

uint8_t srl_direct(uint8_t);
uint8_t srl_direct_hex( uint8_t i );
uint8_t srl_generic(uint8_t);
uint8_t srl_generic_P(uint8_t addr);
uint8_t srl_generic_P_upper(uint8_t addr);
uint8_t sendSignature(uint8_t i);
void test_1wire( void );

/* user defined baudrate */
static inline void change_baudrate( uint8_t newbaud ){
	UBRR0H = 0;
	UBRR0L = newbaud;
}

/* baudrate change pending flag. Ignored when 0,
 * otherwise it contains the new baudrate divisor */
extern uint8_t new_baudrate;

extern uint8_t PROGMEM TABLES_TO_DUMP1[];
extern uint8_t PROGMEM TABLES_TO_DUMP2[];

// Use for debugging
void put_char(uint8_t byte);

/// used  in sipr.c
uint8_t comm_getLambdaView(uint8_t val);
uint8_t comm_setLambdaView(uint8_t val);
#endif
